/*
  Demo 3 – Telemetry: Receive Data Wirelessly and Datalogging
  For Instructable: Arduino Data Logging Shield With Real Time Clock Timestamp ... and Telemetry, Sensor Network
  Link: https://www.instructables.com/Arduino-Data-Logging-Shield-With-Real-Time-Clock-T/
  Youtube video demonstration: https://youtu.be/_Q2pBEgoMN0
  By JD_K April 2023

** This is the sketch for the CLIENT that SENDS data to the server

  Uses ARduino, nRF24L01, thermistor, etc.
*/

// Declaring global variables
const int tempPin = 0; // Analog pin, connection between thermistor and 10 kOhm
unsigned long prevMillis = 9999; // starting off at a high number to trigger the 1st measurement in the loop()
const unsigned long millisInterval = 10000  ; // 1000 = 1 sec; 600000 = 10min

// ***  Code for the nRF24L01_ based on the RadioHead library - nrf24_reliable_datagram_client

#include <RHReliableDatagram.h>
// already included SPI.h
#include <RH_NRF24.h>

// CLIENT_ADDRESS is the address of THIS Arduino which will be sending a message
#define CLIENT_ADDRESS 2
// SERVER_ADDRESS is the address of the Arduino receiving the message below
#define SERVER_ADDRESS 1

// Singleton instance of the radio driver
RH_NRF24 driver;

// Class to manage message delivery and receipt, using the driver declared above
RHReliableDatagram manager(driver, CLIENT_ADDRESS);
uint8_t buf[RH_NRF24_MAX_MESSAGE_LEN];


void setup() {
  Serial.begin(9600);


  // *** nRF24L01 code
  Serial.println(F("initiallizing nRF24L01"));
  if (!manager.init())
    Serial.println(F(" nRF24L01 init failed"));
  // Defaults after init are 2.402 GHz (channel 2), 2Mbps, 0dBm
  // *** End the nRF24L01 code for the set up

}

void loop()
{
  if ((millis() - prevMillis > millisInterval) || (millis() < prevMillis)) {
    prevMillis = millis();
    
  // Code from the Elegoo Complete Starter Kit
  int tempReading = analogRead(tempPin); // tempPin
  double tempK = log(10000.0 * ((1024.0 / tempReading - 1)));
  tempK = 1 / (0.001129148 + (0.000234125 + (0.0000000876741 * tempK * tempK )) * tempK );       //  Temp Kelvin
  float tempC = tempK - 273.15;            // Convert Kelvin to Celcius

  Serial.println(tempC); // for troubleshooting
  Serial.println();


  // This part is from STRINGS for Video from Kevin Durah, 7th Lesson: https://www.youtube.com/watch?v=tCJgnBI1D5o
  char floatValue[5];
  /* tempC is a float but we will need it to be a string to be able to transmit it. So we declare a character array (string) called floatValue;
   five bytes long = a float takes four bytes plus the NULL
  */

  dtostrf(tempC, 1, 2, floatValue);
  /* convert float to string, because "sprintf()" funcion does not work for a float, there is a %f apparently but it does not work
    in the dtostrf() function: the "1" is the minimum size, i.e. as in minimum characters and/or spaces; the "2" is the precision i.e. two decimal places eventhough the original had more
    NOTE: if the length of the floatValue[] or the number of decimal places (the "2" in this case) are not correct, it affects any the OTHER numbers in the data string
  */

  char data[10];
  /* In this sketch, the floatValue character array was created for the purpose of converting the float to the string, and now we are saving it to another string called data,
    which will be transmitted wirelessly;. This is redundant (making a string and saving it to a string), but is left here for future examples use if multiple values of
    different variable types (other floats, integers, etc.) were to be saved to the "data" array using the sprintf line below
    In a previous project using this method, when there were 5 integers and the floatValue string in the sprintf line, the minimum was 20 elements (data[20])
    if there are two decimal places for floatValue.
    Note sure if this is how it works: 2 bytes for each integer, 5 for each comma, 4 for the float, and 1 for the NULL
  */

  sprintf(data, "%s", floatValue); // only converting the string "floatValue" to the string "data"
  /*using sprintf() function to create a string, this line can be used to convert many values into the string called "data"
    %b is for a byte; %s = a string (if wanting a keyword), %i = integer, %li = long integer, %s = a string again
    For example creating a string with 3 types of variables (make sure the data array is large enough): sprintf(data, "%i", "%b", "%s", integerVariableName, byteVariableName, floatValue);
  */

  Serial.print("data string to be sent out: ");
  Serial.println(data); // print it all out as one string to show that it worked

  // *** nRF24L01 code in the Loop
  Serial.println(F("Sending to nrf24_reliable_datagram_server"));
  // Send a message to manager_server
  if (manager.sendtoWait(data, sizeof(data), SERVER_ADDRESS))
  {
    // Now wait for a reply from the server
    uint8_t len = sizeof(buf);
    uint8_t from;
    if (manager.recvfromAckTimeout(buf, &len, 2000, &from))
    {
      Serial.print(F("got reply from : 0x"));
      Serial.print(from, HEX);
      Serial.print(F(": "));
      Serial.println((char*)buf);
    }
    else
    {
      Serial.println(F("No reply, is nrf24_reliable_datagram_server running?"));
    }
  }
  else
  {
    Serial.println(F("sendtoWait failed"));
      }
}
  // *** End of the nRF24L01 code in the Loop
}
